"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.autoCssModules = void 0;
const promises_1 = require("node:fs/promises");
const parser = __importStar(require("@babel/parser"));
const traverse_1 = __importDefault(require("@babel/traverse"));
const t = __importStar(require("@babel/types"));
const generator_1 = __importDefault(require("@babel/generator"));
const node_path_1 = require("node:path");
function judgeTypeScript(path) {
    return path.endsWith('.ts') || path.endsWith('.tsx');
}
const CSS_EXT_NAMES = ['.css', '.less', '.sass', '.scss', '.stylus', '.styl'];
function parseValue(value) {
    const [name, query] = value.split('?');
    return { name, query };
}
const stylesRegexp = new RegExp(`['"](\.+)\\.(${CSS_EXT_NAMES.map((s) => s.substring(1)).join('|')})(\\?[^'"]*)?['"]`);
const autoCssModules = (opts = {}) => {
    function getValue(name, query) {
        return `${name}?${opts.flag || 'modules'}${query ? `&${query}` : ''}`;
    }
    return {
        name: 'auto-css-modules',
        setup(build) {
            build.onLoad({ filter: opts.filter || /\.([tj]sx?)$/ }, (args) => __awaiter(this, void 0, void 0, function* () {
                const isTs = judgeTypeScript(args.path);
                const fileContent = yield (0, promises_1.readFile)(args.path, 'utf-8');
                /**
                 * check if the file contains css import
                 */
                if (!stylesRegexp.test(fileContent)) {
                    return null;
                }
                if (opts.ignore) {
                    if ('function' === typeof opts.ignore && opts.ignore(args.path)) {
                        return null;
                    }
                    if (opts.ignore instanceof RegExp && opts.ignore.test(args.path)) {
                        return null;
                    }
                    throw new Error('ignore option must be a function or a regexp');
                }
                const ast = parser.parse(fileContent, {
                    sourceType: 'module',
                    plugins: ['jsx', 'classProperties', 'decorators-legacy', isTs ? 'typescript' : null].filter(Boolean),
                });
                (0, traverse_1.default)(ast, {
                    ImportDeclaration(path) {
                        const { specifiers, source, source: { value }, } = path.node;
                        const { name, query } = parseValue(value);
                        if (specifiers.length && CSS_EXT_NAMES.includes((0, node_path_1.extname)(name))) {
                            source.value = getValue(name, query);
                        }
                    },
                    VariableDeclarator(path) {
                        const { node } = path;
                        // const styles = await import('./index.less');
                        if (t.isAwaitExpression(node.init) &&
                            t.isCallExpression(node.init.argument) &&
                            t.isImport(node.init.argument.callee) &&
                            node.init.argument.arguments.length === 1 &&
                            t.isStringLiteral(node.init.argument.arguments[0])) {
                            const { name, query } = parseValue(node.init.argument.arguments[0].value);
                            if (CSS_EXT_NAMES.includes((0, node_path_1.extname)(name))) {
                                node.init.argument.arguments[0].value = getValue(name, query);
                            }
                        }
                        // const styles = require('./index.less');
                        else if (t.isCallExpression(node.init) &&
                            t.isIdentifier(node.init.callee) &&
                            node.init.callee.name === 'require' &&
                            node.init.arguments.length === 1 &&
                            t.isStringLiteral(node.init.arguments[0])) {
                            const { name, query } = parseValue(node.init.arguments[0].value);
                            if (CSS_EXT_NAMES.includes((0, node_path_1.extname)(name))) {
                                node.init.arguments[0].value = getValue(name, query);
                            }
                        }
                    },
                });
                let { code, map } = (0, generator_1.default)(ast, {
                    sourceMaps: !!build.initialOptions.sourcemap,
                    sourceRoot: build.initialOptions.sourceRoot,
                    sourceFileName: args.path,
                }, fileContent);
                if (build.initialOptions.sourcemap && map) {
                    code += `\n//# sourceMappingURL=data:application/json;charset=utf-8;base64,${Buffer.from(JSON.stringify(map)).toString('base64')}`;
                }
                const ext = (0, node_path_1.extname)(args.path);
                return { contents: code, loader: ext.substring(1) };
            }));
        },
    };
};
exports.autoCssModules = autoCssModules;
